@@ -34,6 +34,7 @@ import java.text.SimpleDateFormat;
 import java.util.Date;
 import java.util.Enumeration;
 import java.util.HashMap;
+import java.util.Properties;
 import java.util.StringTokenizer;
 
 import javax.net.ssl.KeyManager;
@@ -41,6 +42,8 @@ import javax.net.ssl.SSLContext;
 import javax.net.ssl.TrustManager;
 
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 // Imports required to "implement" Tomcat 7 Interface
 import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.mozilla.jss.CertDatabaseException;
@@ -60,6 +63,8 @@ public class JSSSocketFactory implements
         org.apache.tomcat.util.net.ServerSocketFactory,
         org.apache.tomcat.util.net.SSLUtil {
 
+    static Log logger = LogFactory.getLog(JSSSocketFactory.class);
+
     private static HashMap<String, Integer> cipherMap = new HashMap<String, Integer>();
     static {
         // SSLv2
@@ -336,6 +341,7 @@ public class JSSSocketFactory implements
     }
 
     private AbstractEndpoint endpoint;
+    private Properties config;
 
     static org.apache.commons.logging.Log log = org.apache.commons.logging.LogFactory
             .getLog(JSSSocketFactory.class);
@@ -361,6 +367,11 @@ public class JSSSocketFactory implements
         this.endpoint = endpoint;
     }
 
+    public JSSSocketFactory(AbstractEndpoint endpoint, Properties config) {
+        this.endpoint = endpoint;
+        this.config = config;
+    }
+
     private void debugWrite(String m) throws IOException {
         if (debug) {
             String timeStamp = timeStampFormat.format(new Date());
@@ -370,14 +381,18 @@ public class JSSSocketFactory implements
     }
 
     public void setSSLCiphers(String attr) throws SocketException, IOException {
-        String ciphers = getEndpointAttribute(attr);
+        String ciphers = getProperty(attr);
         if (StringUtils.isEmpty(ciphers)) {
             debugWrite("JSSSocketFactory setSSLCiphers: " + attr + " not found");
             return;
         }
-        StringTokenizer st = new StringTokenizer(ciphers, ",");
+
+        logger.debug("Processing " + attr + ":");
+        StringTokenizer st = new StringTokenizer(ciphers, ", ");
         while (st.hasMoreTokens()) {
             String cipherstr = st.nextToken();
+            logger.debug(" - " + cipherstr);
+
             int cipherid = 0;
             String text;
             boolean state;
@@ -444,14 +459,18 @@ public class JSSSocketFactory implements
      * parameter is ignored.
      */
     public void setSSLOptions() throws SocketException, IOException {
-        String options = getEndpointAttribute("sslOptions");
+        String options = getProperty("sslOptions");
         if (StringUtils.isEmpty(options)) {
             debugWrite("no sslOptions specified");
             return;
         }
-        StringTokenizer st = new StringTokenizer(options, ",");
+
+        logger.debug("Processing sslOptions:");
+        StringTokenizer st = new StringTokenizer(options, ", ");
         while (st.hasMoreTokens()) {
             String option = st.nextToken();
+            logger.debug(" - " + option);
+
             StringTokenizer st1 = new StringTokenizer(option, "=");
             String name = st1.nextToken();
             String value = st1.nextToken();
@@ -555,17 +574,21 @@ public class JSSSocketFactory implements
         return -1;
     }
 
-    String getEndpointAttribute(String tag) {
-        try {
-            return (String) endpoint.getAttribute(tag);
-        } catch (Exception e) {
-            // old tomcat throws an exception if the parameter does not exist
+    String getProperty(String tag) {
+
+        // check <catalina.base>/conf/server.xml
+        String value = (String)endpoint.getAttribute(tag);
+
+        // if not available, check <catalina.base>/conf/tomcatjss.conf
+        if (value == null) {
+            value = config.getProperty(tag);
         }
-        return null;
+
+        return value;
     }
 
-    String getEndpointAttribute(String tag, String defaultValue) {
-        String value = getEndpointAttribute(tag);
+    String getProperty(String tag, String defaultValue) {
+        String value = getProperty(tag);
         if (value == null) {
             return defaultValue;
         }
@@ -574,7 +597,7 @@ public class JSSSocketFactory implements
 
     void init() throws IOException {
         // debug enabled?
-        String deb = getEndpointAttribute("debug");
+        String deb = getProperty("debug");
         if (StringUtils.equals(deb, "true")) {
             debug = true;
             debugFile = new FileWriter("/tmp/tomcatjss.log", true);
@@ -602,14 +625,14 @@ public class JSSSocketFactory implements
 
             // MUST look for "clientauth" (ALL lowercase) since "clientAuth"
             // (camel case) has already been processed by Tomcat 7
-            String clientAuthStr = getEndpointAttribute("clientauth");
+            String clientAuthStr = getProperty("clientauth");
             if (clientAuthStr == null) {
                 debugWrite("JSSSocketFactory init - \"clientauth\" not found, default to want.");
                 clientAuthStr = "want";
             }
             File file = null;
             try {
-                mServerCertNickPath = getEndpointAttribute("serverCertNickFile");
+                mServerCertNickPath = getProperty("serverCertNickFile");
                 if (mServerCertNickPath == null) {
                     throw new IOException("serverCertNickFile not specified");
                 }
@@ -645,7 +668,7 @@ public class JSSSocketFactory implements
                         "JSSSocketFactory: no serverCertNickFile defined");
             }
 
-            // serverCertNick = (String)getEndpointAttribute("serverCert");
+            // serverCertNick = (String)getProperty("serverCert");
             if (clientAuthStr.equalsIgnoreCase("true")
                     || clientAuthStr.equalsIgnoreCase("yes")) {
                 requireClientAuth = true;
@@ -659,7 +682,7 @@ public class JSSSocketFactory implements
                     && ocspConfigured == false) {
                 debugWrite("JSSSocketFactory init - checking for OCSP settings. \n");
                 boolean enableOCSP = false;
-                String doOCSP = getEndpointAttribute("enableOCSP");
+                String doOCSP = getProperty("enableOCSP");
 
                 debugWrite("JSSSocketFactory init - doOCSP flag:" + doOCSP + " \n");
 
@@ -671,10 +694,10 @@ public class JSSSocketFactory implements
                         + "\n");
 
                 if (enableOCSP == true) {
-                    String ocspResponderURL = getEndpointAttribute("ocspResponderURL");
+                    String ocspResponderURL = getProperty("ocspResponderURL");
                     debugWrite("JSSSocketFactory init - ocspResponderURL "
                             + ocspResponderURL + "\n");
-                    String ocspResponderCertNickname = getEndpointAttribute(
+                    String ocspResponderCertNickname = getProperty(
                             "ocspResponderCertNickname");
                     debugWrite("JSSSocketFactory init - ocspResponderCertNickname"
                             + ocspResponderCertNickname + "\n");
@@ -689,9 +712,9 @@ public class JSSSocketFactory implements
                             int ocspMinCacheEntryDuration_i = 3600;
                             int ocspMaxCacheEntryDuration_i = 86400;
 
-                            String ocspCacheSize = getEndpointAttribute("ocspCacheSize");
-                            String ocspMinCacheEntryDuration = getEndpointAttribute("ocspMinCacheEntryDuration");
-                            String ocspMaxCacheEntryDuration = getEndpointAttribute("ocspMaxCacheEntryDuration");
+                            String ocspCacheSize = getProperty("ocspCacheSize");
+                            String ocspMinCacheEntryDuration = getProperty("ocspMinCacheEntryDuration");
+                            String ocspMaxCacheEntryDuration = getProperty("ocspMaxCacheEntryDuration");
 
                             if (ocspCacheSize != null
                                     || ocspMinCacheEntryDuration != null
@@ -718,7 +741,7 @@ public class JSSSocketFactory implements
                             }
 
                             // defualt to 60 seconds;
-                            String ocspTimeout = getEndpointAttribute("ocspTimeout");
+                            String ocspTimeout = getProperty("ocspTimeout");
                             if (ocspTimeout != null) {
                                 debugWrite("JSSSocketFactory init - ocspTimeout= \n" + ocspTimeout);
                                 int ocspTimeout_i = Integer.parseInt(ocspTimeout);
@@ -749,7 +772,7 @@ public class JSSSocketFactory implements
             // 12 hours = 43200 seconds
             SSLServerSocket.configServerSessionIDCache(0, 43200, 43200, null);
 
-            String strictCiphersStr = getEndpointAttribute("strictCiphers");
+            String strictCiphersStr = getProperty("strictCiphers");
             if (StringUtils.equalsIgnoreCase(strictCiphersStr, "true")
                     || StringUtils.equalsIgnoreCase(strictCiphersStr, "yes")) {
                 mStrictCiphers = true;
@@ -762,7 +785,7 @@ public class JSSSocketFactory implements
                 debugWrite("SSSocketFactory init - before setSSLCiphers, strictCiphers is false\n");
             }
 
-            String sslVersionRangeStream = getEndpointAttribute("sslVersionRangeStream");
+            String sslVersionRangeStream = getProperty("sslVersionRangeStream");
             if ((sslVersionRangeStream != null)
                     && !sslVersionRangeStream.equals("")) {
                 debugWrite("SSSocketFactory init - calling setSSLVersionRangeDefault() for type STREAM\n");
@@ -772,7 +795,7 @@ public class JSSSocketFactory implements
                 debugWrite("SSSocketFactory init - after setSSLVersionRangeDefault() for type STREAM\n");
             }
 
-            String sslVersionRangeDatagram = getEndpointAttribute("sslVersionRangeDatagram");
+            String sslVersionRangeDatagram = getProperty("sslVersionRangeDatagram");
             if ((sslVersionRangeDatagram != null)
                     && !sslVersionRangeDatagram.equals("")) {
                 debugWrite("SSSocketFactory init - calling setSSLVersionRangeDefault() for type DATA_GRAM\n");
@@ -843,11 +866,11 @@ public class JSSSocketFactory implements
 
     private void initializePasswordStore() throws InstantiationException, IllegalAccessException,
             ClassNotFoundException, IOException {
-        mPwdClass = getEndpointAttribute("passwordClass");
+        mPwdClass = getProperty("passwordClass");
         if (mPwdClass == null) {
             throw new IOException("Misconfiguration: passwordClass is not defined");
         }
-        mPwdPath = getEndpointAttribute("passwordFile");
+        mPwdPath = getProperty("passwordFile");
 
         mPasswordStore = (IPasswordStore) Class.forName(mPwdClass).newInstance();
         debugWrite("JSSSocketFactory init - password reader initialized\n");
@@ -858,7 +881,7 @@ public class JSSSocketFactory implements
 
     private CryptoManager getCryptoManager() throws KeyDatabaseException, CertDatabaseException,
             GeneralSecurityException, NotInitializedException, IOException {
-        String certDir = getEndpointAttribute("certdbDir");
+        String certDir = getProperty("certdbDir");
         if (certDir == null) {
             throw new IOException("Misconfiguration: certdir not defined");
         }
@@ -923,6 +946,10 @@ public class JSSSocketFactory implements
         SSLSocket asock = null;
         try {
             asock = (SSLSocket) socket.accept();
+
+            TomcatJSS tomcatjss = TomcatJSS.getInstance();
+            asock.addSocketListener(tomcatjss);
+
             if (wantClientAuth || requireClientAuth) {
                 asock.requestClientAuth(true);
                 if (requireClientAuth == true) {
